from sims4.tuning.instances import HashedTunedInstanceMetaclass
from sims4.tuning.tunable import HasTunableReference, TunableList, TunableReference, TunableTuple
from sims4.resources import Types
from services import get_instance_manager
from snippets import TunableAffordanceFilterSnippet
from Pandasama_Childbirth import logger

class ObjectPartInjector(HasTunableReference, metaclass=HashedTunedInstanceMetaclass,
                         manager=get_instance_manager(Types.SNIPPET)):
    INSTANCE_TUNABLES = {
        "injections": TunableList(
            description="Inject an affordance filter into many object parts' supported_affordance_data",
            tunable=TunableTuple(
                object_parts=TunableList(description="A list of object parts this filter should be injected to",
                                         tunable=TunableReference(manager=get_instance_manager(Types.OBJECT_PART))),
                filter=TunableAffordanceFilterSnippet(
                    description="Affordance filter to merge with object part supported_affordance_data"),
            )
        )
    }


def panda_on_snippets_loaded(*args, **kwargs):
    try:
        snippet_manager = get_instance_manager(Types.SNIPPET)
        snippets = snippet_manager.get_ordered_types(only_subclasses_of=ObjectPartInjector)
        # loop through all available injector snippets
        for snippet in snippets:
            # loop through all injections
            for row in snippet.injections:
                # loop through all object parts
                for objpart in row.object_parts:
                    try:
                        # skip obj parts that failed to load
                        if objpart is None:
                            continue
                        # merge the current row's filter into each object part
                        compat = objpart.supported_affordance_data.compatibility
                        new_included_list = row.filter._tuned_values.default_inclusion.include_affordances + compat._tuned_values.default_inclusion.include_affordances
                        new_excluded_list = row.filter._tuned_values.default_inclusion.exclude_affordances + compat._tuned_values.default_inclusion.exclude_affordances

                        new_default_inclusion = compat._tuned_values.default_inclusion.clone_with_overrides(
                            include_affordances=new_included_list, exclude_affordances=new_excluded_list)
                        compat._tuned_values = compat._tuned_values.clone_with_overrides(
                            default_inclusion=new_default_inclusion)
                    except:
                        logger.exception("failed injection to object part: {}".format(objpart))
    except:
        logger.exception("Failed injecting to object part snippets")


get_instance_manager(Types.SNIPPET).add_on_load_complete(panda_on_snippets_loaded)